Package: EnMCB
Type: Package
Title: Predicting Disease Progression Based on Methylation Correlated
        Blocks using Ensemble Models
Version: 1.22.0
Date: 2023-3-13
Author: Xin Yu 
Maintainer: Xin Yu <whirlsyu@gmail.com>
Depends: R (>= 4.0)
Encoding: UTF-8
Imports: survivalROC, glmnet, rms, mboost, Matrix, igraph, methods,
        survivalsvm, ggplot2, boot, e1071, survival, BiocFileCache
ImportFrom:
VignetteBuilder: knitr
Suggests: SummarizedExperiment, testthat, Biobase, survminer,
        affycoretools, knitr, plotROC, limma, rmarkdown
Description: Creation of the correlated blocks using DNA methylation profiles. 
    Machine learning models can be constructed to predict differentially methylated blocks and disease progression.
License: GPL-2
BugReports: https://github.com/whirlsyu/EnMCB/issues
biocViews: Normalization, DNAMethylation, MethylationArray,
        SupportVectorMachine
LazyData: FALSE
RoxygenNote: 7.2.3
git_url: https://git.bioconductor.org/packages/EnMCB
git_branch: RELEASE_3_22
git_last_commit: d87a235
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:47:14 UTC; biocbuild
