# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,ridgemat)
S3method(predict,mcb.coxph.penal)
export(CompareMCB)
export(DiffMCB)
export(IdentifyMCB)
export(IdentifyMCB_parallel)
export(as.ridgemat)
export(create_demo)
export(draw_survival_curve)
export(ensemble_model)
export(ensemble_prediction)
export(fast_roc_calculation)
export(metricMCB)
export(metricMCB.cv)
export(multi_coxph)
export(pre_process_methylation)
export(univ_coxph)
import(BiocFileCache)
import(ggplot2)
import(glmnet)
import(stats)
import(survival)
import(survivalROC)
import(survivalsvm)
import(utils)
importFrom(mboost,CoxPH)
importFrom(mboost,boost_control)
importFrom(mboost,glmboost)
importFrom(mboost,predict.glmboost)
importFrom(methods,as)
