% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{as.ridgemat}
\alias{as.ridgemat}
\title{ridge matrix}
\usage{
as.ridgemat(x)
}
\arguments{
\item{x}{data vector}
}
\description{
as.matrix attempts to turn its argument
}
